@@ -1,35 +1,35 @@
 """
 Class OWTextableRecode
-Copyright 2012-2017 LangTech Sarl (info@langtech.ch)
+Copyright 2012-2019 LangTech Sarl (info@langtech.ch)
 -----------------------------------------------------------------------------
-This file is part of the Orange-Textable package v3.0.
+This file is part of the Orange3-Textable package.
 
-Orange-Textable v3.0 is free software: you can redistribute it and/or modify
+Orange3-Textable is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.
 
-Orange-Textable v3.0 is distributed in the hope that it will be useful,
+Orange3-Textable is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with Orange-Textable v3.0. If not, see <http://www.gnu.org/licenses/>.
+along with Orange3-Textable. If not, see <http://www.gnu.org/licenses/>.
 """
 
-__version__ = '0.13.4'
+__version__ = '0.13.9'
 
 import os, re, codecs, json
 
-from PyQt4.QtGui import QFont
-from PyQt4.QtGui import QFileDialog, QMessageBox
+from PyQt5.QtGui import QFont
+from PyQt5.QtWidgets import QFileDialog, QMessageBox
 
 import LTTL.Segmenter as Segmenter
 from LTTL.Segmentation import Segmentation
 
 from .TextableUtils import (
-    OWTextableBaseWidget, VersionedSettingsHandler, JSONMessage,
+    OWTextableBaseWidget, VersionedSettingsHandler, JSONMessage, ProgressBar,
     InfoBox, SendButton, AdvancedSettings, pluralize, normalizeCarriageReturns
 )
 
@@ -55,7 +55,6 @@ class OWTextableRecode(OWTextableBaseWidget):
         version=__version__.rsplit(".", 1)[0]
     )
     # Settings...
-    autoSend = settings.Setting(True)
     substitutions = settings.Setting([])
     copyAnnotations = settings.Setting(True)
     displayAdvancedSettings = settings.Setting(False)
@@ -506,12 +505,14 @@ class OWTextableRecode(OWTextableBaseWidget):
         # Perform recoding...
         self.clearCreatedInputIndices()
         previousNumInputs = len(Segmentation.data)
-        progressBar = gui.ProgressBar(
+        self.infoBox.setText(u"Processing, please wait...", "warning")
+        self.controlArea.setDisabled(True)
+        progressBar = ProgressBar(
             self,
             iterations=len(self.segmentation)
         )
         try:
-            recoded_data = Segmenter.recode(
+            recoded_data, num_subs = Segmenter.recode(
                 segmentation=self.segmentation,
                 substitutions=substitutions,
                 label=self.captionTitle,
@@ -520,29 +521,32 @@ class OWTextableRecode(OWTextableBaseWidget):
             )
             newNumInputs = len(Segmentation.data)
             self.createdInputIndices = range(previousNumInputs, newNumInputs)
-            message = u'%i segment@p sent to output.' % len(recoded_data)
+            message = u'%i segment@p sent to output' % len(recoded_data)
             message = pluralize(message, len(recoded_data))
+            if num_subs:
+                message += u' (%i replacement@p performed).' % num_subs
+                message = pluralize(message, num_subs)
+            else:
+                message += u" (no replacements performed)."
             self.infoBox.setText(message)
             self.send('Recoded data', recoded_data, self)
         except re.error as re_error:
-            if re_error.message == 'invalid group reference':
-                self.infoBox.setText(
-                    u'reference to unmatched group in annotation key ' +    \
-                    u'and/or value.',
-                    'error'
-                )
-            else:
-                try:
-                    message = u'Please enter a valid regex (error: %s).' %    \
-                              re_error.msg
-                except AttributeError:
-                    message = u'Please enter a valid regex.'
-                self.infoBox.setText(
-                    message, 'error'
-                )
+            try:
+                if str(re_error) == 'invalid group reference':
+                    message = u'Reference to unmatched group in ' +   \
+                              u' annotation key and/or value.'
+                else:
+                    message = u'Please enter a valid regex (error: %s).' %   \
+                              str(re_error)
+            except AttributeError:
+                message = u'Please enter a valid regex.'
+            self.infoBox.setText(
+                message, 'error'
+            )
             self.send('Recoded data', None, self)
         self.sendButton.resetSettingsChangedFlag()
         progressBar.finish()
+        self.controlArea.setDisabled(False)
 
     def inputData(self, segmentation):
         """Process incoming segmentation"""
@@ -552,7 +556,7 @@ class OWTextableRecode(OWTextableBaseWidget):
 
     def importList(self):
         """Display a FileDialog and import substitution list"""
-        filePath = QFileDialog.getOpenFileName(
+        filePath, _ = QFileDialog.getOpenFileName(
             self,
             u'Import Regex List',
             self.lastLocation,
@@ -630,7 +634,7 @@ class OWTextableRecode(OWTextableBaseWidget):
                 toDump[-1]['multiline'] = substitution[4]
             if substitution[5]:
                 toDump[-1]['dot_all'] = substitution[5]
-        filePath = QFileDialog.getSaveFileName(
+        filePath, _ = QFileDialog.getSaveFileName(
             self,
             u'Export Substitution List',
             self.lastLocation,
@@ -661,10 +665,10 @@ class OWTextableRecode(OWTextableBaseWidget):
         if self.selectedSubstLabels:
             index = self.selectedSubstLabels[0]
             if index > 0:
-                temp = self.substitutions[index - 1]
-                self.substitutions[index - 1] = self.substitutions[index]
+                temp = self.substitutions[index-1]
+                self.substitutions[index-1] = self.substitutions[index]
                 self.substitutions[index] = temp
-                self.selectedSubstLabels.listBox.item(index - 1).setSelected(1)
+                self.selectedSubstLabels = [index-1]
                 self.sendButton.settingsChanged()
 
     def moveDown(self):
@@ -672,10 +676,10 @@ class OWTextableRecode(OWTextableBaseWidget):
         if self.selectedSubstLabels:
             index = self.selectedSubstLabels[0]
             if index < len(self.substitutions) - 1:
-                temp = self.substitutions[index + 1]
-                self.substitutions[index + 1] = self.substitutions[index]
+                temp = self.substitutions[index+1]
+                self.substitutions[index+1] = self.substitutions[index]
                 self.substitutions[index] = temp
-                self.selectedSubstLabels.listBox.item(index + 1).setSelected(1)
+                self.selectedSubstLabels = [index+1]
                 self.sendButton.settingsChanged()
 
     def clearAll(self):
@@ -741,9 +745,7 @@ class OWTextableRecode(OWTextableBaseWidget):
             self.substLabels = self.substLabels
             if cachedLabel is not None:
                 self.sendButton.sendIfPreCallback = None
-                self.selectedSubstLabels.listBox.item(
-                    cachedLabel
-                ).setSelected(1)
+                self.selectedSubstLabels = [cachedLabel]
                 self.sendButton.sendIfPreCallback = self.updateGUI
             if self.newRegex:
                 self.addButton.setDisabled(False)
@@ -792,7 +794,7 @@ class OWTextableRecode(OWTextableBaseWidget):
 
 if __name__ == '__main__':
     import sys
-    from PyQt4.QtGui import QApplication
+    from PyQt5.QtWidgets import QApplication
     appl = QApplication(sys.argv)
     ow = OWTextableRecode()
     ow.show()
