@@ -1,40 +1,46 @@
 """
 Class OWTextableTextFiles
-Copyright 2012-2017 LangTech Sarl (info@langtech.ch)
+Copyright 2012-2019 LangTech Sarl (info@langtech.ch)
 -----------------------------------------------------------------------------
-This file is part of the Orange-Textable package v3.0.
+This file is part of the Orange3-Textable package.
 
-Orange-Textable v3.0 is free software: you can redistribute it and/or modify
+Orange3-Textable is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.
 
-Orange-Textable v3.0 is distributed in the hope that it will be useful,
+Orange3-Textable is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with Orange-Textable v3.0. If not, see <http://www.gnu.org/licenses/>.
+along with Orange3-Textable. If not, see <http://www.gnu.org/licenses/>.
 """
 
-__version__ = '0.17.4'
+__version__ = '0.17.10'
 
 
-import codecs, io, os, re, json
+import codecs
+import os
+import re
+import json
 from unicodedata import normalize
 
-from PyQt4.QtCore import QTimer
-from PyQt4.QtGui import QFileDialog, QMessageBox
-from PyQt4.QtGui import QFont
+from PyQt5.QtCore import QTimer
+from PyQt5.QtGui import QFont
+from PyQt5.QtWidgets import QFileDialog, QMessageBox
+
+from chardet.universaldetector import UniversalDetector
 
 from LTTL.Segmentation import Segmentation
 from LTTL.Input import Input
 import LTTL.Segmenter as Segmenter
 
 from .TextableUtils import (
-    OWTextableBaseWidget, VersionedSettingsHandler,
+    OWTextableBaseWidget, VersionedSettingsHandler, ProgressBar,
     JSONMessage, InfoBox, SendButton, AdvancedSettings,
+    addSeparatorAfterDefaultEncodings, addAutoDetectEncoding,
     getPredefinedEncodings, normalizeCarriageReturns, pluralize
 )
 
@@ -63,9 +69,8 @@ class OWTextableTextFiles(OWTextableBaseWidget):
     )
 
     # Settings...
-    autoSend = settings.Setting(True)
     files = settings.Setting([])
-    encoding = settings.Setting('iso-8859-1')
+    encoding = settings.Setting('(auto-detect)')
     autoNumber = settings.Setting(False)
     autoNumberKey = settings.Setting(u'num')
     importFilenames = settings.Setting(True)
@@ -143,7 +148,7 @@ class OWTextableTextFiles(OWTextableBaseWidget):
             ),
         )
         gui.separator(widget=basicFileBox, width=3)
-        gui.comboBox(
+        advancedEncodingsCombobox = gui.comboBox(
             widget=basicFileBox,
             master=self,
             value='encoding',
@@ -157,6 +162,8 @@ class OWTextableTextFiles(OWTextableBaseWidget):
                 u"Select input file(s) encoding."
             ),
         )
+        addSeparatorAfterDefaultEncodings(advancedEncodingsCombobox)
+        addAutoDetectEncoding(advancedEncodingsCombobox)
         gui.separator(widget=basicFileBox, width=3)
         self.advancedSettings.basicWidgets.append(basicFileBox)
         self.advancedSettings.basicWidgetsAppendSeparator()
@@ -305,7 +312,7 @@ class OWTextableTextFiles(OWTextableBaseWidget):
             ),
         )
         gui.separator(widget=addFileBox, width=3)
-        gui.comboBox(
+        basicEncodingsCombobox = gui.comboBox(
             widget=addFileBox,
             master=self,
             value='encoding',
@@ -319,6 +326,9 @@ class OWTextableTextFiles(OWTextableBaseWidget):
                 u"Select input file(s) encoding."
             ),
         )
+        addSeparatorAfterDefaultEncodings(basicEncodingsCombobox)
+        addAutoDetectEncoding(basicEncodingsCombobox)
+        self.encoding = self.encoding
         gui.separator(widget=addFileBox, width=3)
         gui.lineEdit(
             widget=addFileBox,
@@ -520,7 +530,9 @@ class OWTextableTextFiles(OWTextableBaseWidget):
         else:
             myFiles = [[self.file, self.encoding, u'', u'']]
 
-        progressBar = gui.ProgressBar(
+        self.infoBox.setText(u"Processing, please wait...", "warning")
+        self.controlArea.setDisabled(True)
+        progressBar = ProgressBar(
             self,
             iterations=len(myFiles)
         )
@@ -529,14 +541,27 @@ class OWTextableTextFiles(OWTextableBaseWidget):
         for myFile in myFiles:
             filePath = myFile[0]
             encoding = myFile[1]
+            encoding = re.sub(r"[ ]\(.+", "", encoding)
             annotation_key = myFile[2]
             annotation_value = myFile[3]
 
             # Try to open the file...
             self.error()
             try:
-                # simply open in Python 3?
-                fh = io.open(filePath, mode='rU', encoding=encoding)
+                if encoding == "(auto-detect)":
+                    detector = UniversalDetector()
+                    fh = open(filePath, 'rb')
+                    for line in fh:
+                        detector.feed(line)
+                        if detector.done: break
+                    detector.close()
+                    fh.close()
+                    encoding = detector.result['encoding']
+                fh = open(
+                    filePath,
+                    mode='rU',
+                    encoding=encoding,
+                )
                 try:
                     fileContent = ""
                     i = 0
@@ -559,6 +584,7 @@ class OWTextableTextFiles(OWTextableBaseWidget):
                         message = u"Please select another encoding."
                     self.infoBox.setText(message, 'error')
                     self.send('Text data', None, self)
+                    self.controlArea.setDisabled(False)
                     return
                 finally:
                     fh.close()
@@ -570,6 +596,7 @@ class OWTextableTextFiles(OWTextableBaseWidget):
                     message = u"Couldn't open file."
                 self.infoBox.setText(message, 'error')
                 self.send('Text data', None, self)
+                self.controlArea.setDisabled(False)
                 return
 
             # Remove utf-8 BOM if necessary...
@@ -635,6 +662,7 @@ class OWTextableTextFiles(OWTextableBaseWidget):
         message = pluralize(message, numChars)
         self.infoBox.setText(message)
         progressBar.finish()
+        self.controlArea.setDisabled(False)
 
         self.send('Text data', self.segmentation, self)
         self.sendButton.resetSettingsChangedFlag()
@@ -646,7 +674,7 @@ class OWTextableTextFiles(OWTextableBaseWidget):
 
     def importList(self):
         """Display a FileDialog and import file list"""
-        filePath = QFileDialog.getOpenFileName(
+        filePath, _ = QFileDialog.getOpenFileName(
             self,
             u'Import File List',
             self.lastLocation,
@@ -715,7 +743,7 @@ class OWTextableTextFiles(OWTextableBaseWidget):
             if myfile[2] and myfile[3]:
                 toDump[-1]['annotation_key'] = myfile[2]
                 toDump[-1]['annotation_value'] = myfile[3]
-        filePath =QFileDialog.getSaveFileName(
+        filePath, _ = QFileDialog.getSaveFileName(
             self,
             u'Export File List',
             self.lastLocation,
@@ -745,7 +773,7 @@ class OWTextableTextFiles(OWTextableBaseWidget):
     def browse(self):
         """Display a FileDialog and select files"""
         if self.displayAdvancedSettings:
-            filePathList = QFileDialog.getOpenFileNames(
+            filePathList, _ = QFileDialog.getOpenFileNames(
                 self,
                 u'Select Text File(s)',
                 self.lastLocation,
@@ -758,7 +786,7 @@ class OWTextableTextFiles(OWTextableBaseWidget):
             self.lastLocation = os.path.dirname(filePathList[-1])
             self.updateGUI()
         else:
-            filePath = QFileDialog.getOpenFileName(
+            filePath, _ = QFileDialog.getOpenFileName(
                 self,
                 u'Open Text File',
                 self.lastLocation,
@@ -776,10 +804,10 @@ class OWTextableTextFiles(OWTextableBaseWidget):
         if self.selectedFileLabels:
             index = self.selectedFileLabels[0]
             if index > 0:
-                temp = self.files[index - 1]
-                self.files[index - 1] = self.files[index]
+                temp = self.files[index-1]
+                self.files[index-1] = self.files[index]
                 self.files[index] = temp
-                self.selectedFileLabels.listBox.item(index - 1).setSelected(1)
+                self.selectedFileLabels = [index-1]
                 self.sendButton.settingsChanged()
 
     def moveDown(self):
@@ -787,10 +815,10 @@ class OWTextableTextFiles(OWTextableBaseWidget):
         if self.selectedFileLabels:
             index = self.selectedFileLabels[0]
             if index < len(self.files) - 1:
-                temp = self.files[index + 1]
-                self.files[index + 1] = self.files[index]
+                temp = self.files[index+1]
+                self.files[index+1] = self.files[index]
                 self.files[index] = temp
-                self.selectedFileLabels.listBox.item(index + 1).setSelected(1)
+                self.selectedFileLabels = [index+1]
                 self.sendButton.settingsChanged()
 
     def clearAll(self):
@@ -811,9 +839,10 @@ class OWTextableTextFiles(OWTextableBaseWidget):
         """Add files to files attr"""
         filePathList = re.split(r' +/ +', self.newFiles)
         for filePath in filePathList:
+            encoding = re.sub(r"[ ]\(.+", "", self.encoding)
             self.files.append((
                 filePath,
-                self.encoding,
+                encoding,
                 self.newAnnotationKey,
                 self.newAnnotationValue,
             ))
@@ -848,9 +877,7 @@ class OWTextableTextFiles(OWTextableBaseWidget):
             self.fileLabels = self.fileLabels
             if cachedLabel is not None:
                 self.sendButton.sendIfPreCallback = None
-                self.selectedFileLabels.listBox.item(
-                    cachedLabel
-                ).setSelected(1)
+                self.selectedFileLabels = [cachedLabel]
                 self.sendButton.sendIfPreCallback = self.updateGUI
             if self.newFiles:
                 if (
@@ -913,7 +940,7 @@ class OWTextableTextFiles(OWTextableBaseWidget):
 
 if __name__ == '__main__':
     import sys
-    from PyQt4.QtGui import QApplication
+    from PyQt5.QtWidgets import QApplication
     appl = QApplication(sys.argv)
     ow = OWTextableTextFiles()
     ow.show()
